@@ -23,8 +23,8 @@ from sqlalchemy.orm import immediateload, Query, aliased
 
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from neutron_lib.db import api as db
 
-import neutron.db.api as db
 from neutron.db.models import l3 as l3_models
 from neutron.db.models import l3ha as l3ha_models
 from neutron.db.models.plugins.ml2 import vlanallocation
@@ -524,67 +524,3 @@ def get_baremetal_sg_bindings():
                        .group_by(sg_binding_model.port_id)
                        .having(func.count(sg_binding_model.port_id) == 1))
     return sg_bindings
-
-
-def get_subnet_gateway_ipv4(subnet_id):
-    """Returns the IPv4 gateway of the router if exists"""
-    session = db.get_reader_session()
-    with session.begin():
-        subnet_model = models_v2.Subnet
-        router_model = l3_models.Router
-        port_model = models_v2.Port
-        ip_allocation_model = models_v2.IPAllocation
-        result = (session
-                  .query(router_model.name,
-                         router_model.id,
-                         subnet_model.gateway_ip.label('gip'),
-                         subnet_model.cidr,
-                         subnet_model.ip_version,
-                         port_model.network_id,
-                         ip_allocation_model.ip_address)
-                  .filter(and_(router_model.gw_port_id == port_model.id,
-                               port_model.network_id ==
-                               subnet_model.network_id,
-                               ip_allocation_model.network_id ==
-                               subnet_model.network_id,
-                               subnet_model.ip_version == 4,
-                               subnet_model.id == subnet_id)).first())
-    return _format_gateway_result(result)
-
-
-def get_network_gateway_ipv4(network_id):
-    """Returns all the routers and IPv4 gateway that have network as gateway"""
-    session = db.get_reader_session()
-    with session.begin():
-        subnet_model = models_v2.Subnet
-        router_model = l3_models.Router
-        port_model = models_v2.Port
-        ip_allocation_model = models_v2.IPAllocation
-        result = (session
-                  .query(router_model.name,
-                         router_model.id,
-                         subnet_model.gateway_ip.label('gip'),
-                         subnet_model.cidr,
-                         subnet_model.ip_version,
-                         port_model.network_id,
-                         ip_allocation_model.ip_address)
-                  .filter(and_(router_model.gw_port_id == port_model.id,
-                               port_model.network_id ==
-                               subnet_model.network_id,
-                               ip_allocation_model.network_id ==
-                               subnet_model.network_id,
-                               port_model.network_id == network_id,
-                               subnet_model.ip_version == 4)).first())
-    return _format_gateway_result(result)
-
-
-def _format_gateway_result(db_result):
-    """This function formats result as needed by add_router_interface"""
-    if not db_result:
-        return None
-    result = {
-        k: db_result[i]
-        for i, k in enumerate(
-            ('name', 'id', 'gip', 'cidr', 'ip_version', 'network_id',
-             'fixed_ip'))}
-    return result
