@@ -13,8 +13,8 @@
 #    under the License.
 
 import copy
-import traceback
 
+from neutron.common import rpc as n_rpc
 from neutron_lib.agent import topics
 from neutron_lib import constants as n_const
 from neutron_lib import context as nctx
@@ -30,7 +30,6 @@ from oslo_utils import excutils
 
 from neutron.api.rpc.agentnotifiers import l3_rpc_agent_api
 from neutron.api.rpc.handlers import l3_rpc
-from neutron.common import rpc as n_rpc
 from neutron.db import extraroute_db
 from neutron.db import l3_agentschedulers_db
 from neutron.db import l3_gwmode_db
@@ -49,7 +48,6 @@ class AristaL3SyncWorker(worker.BaseWorker):
         self._protected_vlans = driver._protected_vlans
         self._servers = driver._servers
         self._use_vrf = driver._use_vrf
-        self._vrf_default_route = driver._vrf_default_route
         self._loop = None
         super(AristaL3SyncWorker, self).__init__(worker_process_count=0)
 
@@ -88,8 +86,7 @@ class AristaL3SyncWorker(worker.BaseWorker):
                 ctx,
                 filters={
                     'device_id': [r['id']],
-                    'device_owner': [n_const.DEVICE_OWNER_ROUTER_INTF,
-                                     n_const.DEVICE_OWNER_ROUTER_GW]}) or []
+                    'device_owner': [n_const.DEVICE_OWNER_ROUTER_INTF]}) or []
             for p in ports:
                 router_interface = r.copy()
                 net_id = p['network_id']
@@ -101,20 +98,11 @@ class AristaL3SyncWorker(worker.BaseWorker):
                 router_interface['seg_id'] = seg_id
                 router_interface['cidr'] = subnet['cidr']
                 router_interface['gip'] = subnet['gateway_ip']
-                if p['id'] == r['gw_port_id']:
-                    r['gip'] = subnet['gateway_ip']
-                router_interface['fixed_ip'] = p['fixed_ips'][0]['ip_address']
                 router_interface['ip_version'] = subnet['ip_version']
                 router_interface['subnet_id'] = subnet_id
                 router_interfaces.append(router_interface)
         return routers, router_interfaces
 
-    def _get_ext_gateway_ip(self, router):
-        if ('external_gateway_info' not in router or
-                router['external_gateway_info'] is None):
-            return None
-        return router.get('gip')
-
     def synchronize(self):
         """Synchronizes Router DB from Neturon DB with EOS.
 
@@ -125,36 +113,16 @@ class AristaL3SyncWorker(worker.BaseWorker):
         same commands can be repeated.
         """
         LOG.info(_LI('Syncing Neutron Router DB <-> EOS'))
-        try:
-            if self._use_vrf:
-                self.driver._update_vrf_commands()
-
-            routers, router_interfaces = self.get_routers_and_interfaces()
-            if self._enable_cleanup:
-                expected_vrfs = set()
-                expected_vrf_default_routes = set()
-                if self._use_vrf:
-                    expected_vrfs.update(self.driver._arista_router_name(
-                        r['id'], r['name']) for r in routers)
-                    if self._vrf_default_route:
-                        expected_vrf_default_routes.update(
-                            (self.driver._arista_router_name(router['id'],
-                                                             router['name']),
-                             self._get_ext_gateway_ip(router))
-                            for router in routers)
-
-                expected_vlans = set(r['seg_id'] for r in router_interfaces)
-                LOG.info(_LI('Syncing Neutron Router DB - cleanup'))
-                self.do_cleanup(expected_vrfs, expected_vlans,
-                                expected_vrf_default_routes)
-            LOG.info(_LI('Syncing Neutron Router DB - creating routers'))
-            self.create_routers(routers)
-            LOG.info(_LI('Syncing Neutron Router DB - creating interfaces'))
-            self.create_router_interfaces(router_interfaces)
-            LOG.info(_LI('Syncing Neutron Router DB finished'))
-        except Exception:
-            exc_str = traceback.format_exc()
-            LOG.error(_LE("Error during synchronize processing %s"), exc_str)
+        routers, router_interfaces = self.get_routers_and_interfaces()
+        expected_vrfs = set()
+        if self._use_vrf:
+            expected_vrfs.update(self.driver._arista_router_name(
+                r['id'], r['name']) for r in routers)
+        expected_vlans = set(r['seg_id'] for r in router_interfaces)
+        if self._enable_cleanup:
+            self.do_cleanup(expected_vrfs, expected_vlans)
+        self.create_routers(routers)
+        self.create_router_interfaces(router_interfaces)
 
     def get_vrfs(self, server):
         ret = self.driver._run_eos_cmds(['show vrf'], server)
@@ -181,31 +149,7 @@ class AristaL3SyncWorker(worker.BaseWorker):
                         if not info['dynamic'])
         return eos_vlans
 
-    def _get_vrf_default_route(self, vrf, vrf_routes):
-        if (not vrf_routes or
-                'routes' not in vrf_routes or
-                '0.0.0.0/0' not in vrf_routes['routes'] or
-                'vias' not in vrf_routes['routes']['0.0.0.0/0']):
-            return set()
-        result = set()
-        for via in vrf_routes['routes']['0.0.0.0/0']['vias']:
-            result.add((vrf, via.get('nexthopAddr')))
-        return result
-
-    def get_vrf_default_routes(self, server):
-        ret = self.driver._run_eos_cmds(['show ip route vrf all'], server)
-        if len(ret or []) != 1 or 'vrfs' not in ret[0].keys():
-            return set()
-
-        eos_vrf_gateway = set()
-        for vrf, vrf_route in ret[0]['vrfs'].items():
-            if '__OpenStack__' in vrf:
-                eos_vrf_gateway.update(self._get_vrf_default_route(vrf,
-                                                                   vrf_route))
-        return eos_vrf_gateway
-
-    def do_cleanup(self, expected_vrfs, expected_vlans,
-                   expected_vrf_default_routes):
+    def do_cleanup(self, expected_vrfs, expected_vlans):
         for server in self._servers:
             eos_svis = self.get_svis(server)
             eos_vlans = self.get_vlans(server)
@@ -221,18 +165,8 @@ class AristaL3SyncWorker(worker.BaseWorker):
             if self._use_vrf:
                 eos_vrfs = self.get_vrfs(server)
                 vrfs_to_delete = eos_vrfs - expected_vrfs
-                delete_cmds.extend([c.format(vrf)
-                                    for c in self.driver.routerDict['delete']
+                delete_cmds.extend(['no vrf definition %s' % vrf
                                     for vrf in vrfs_to_delete])
-                if self._vrf_default_route:
-                    eos_vrf_gateway = self.get_vrf_default_routes(server)
-
-                    vrf_gw_to_delete = (eos_vrf_gateway -
-                                        expected_vrf_default_routes)
-
-                    delete_cmds.extend(
-                        'no ip route vrf %s 0.0.0.0/0' % vrf
-                        for (vrf, _) in vrf_gw_to_delete)
             if delete_cmds:
                 self.driver._run_config_cmds(delete_cmds, server)
 
@@ -363,13 +297,9 @@ class AristaL3ServicePlugin(service_base.ServicePluginBase,
         add_by_port, add_by_sub = self._validate_interface_info(interface_info)
         if add_by_sub:
             subnet = core.get_subnet(context, interface_info['subnet_id'])
-            # If we add by subnet and we have no port allocated, assigned
-            # gateway IP for the interface
-            fixed_ip = subnet['gateway_ip']
         elif add_by_port:
             port = core.get_port(context, interface_info['port_id'])
             subnet_id = port['fixed_ips'][0]['subnet_id']
-            fixed_ip = port['fixed_ips'][0]['ip_address']
             subnet = core.get_subnet(context, subnet_id)
         network_id = subnet['network_id']
 
@@ -385,7 +315,6 @@ class AristaL3ServicePlugin(service_base.ServicePluginBase,
         router_info['name'] = router['name']
         router_info['cidr'] = subnet['cidr']
         router_info['gip'] = subnet['gateway_ip']
-        router_info['fixed_ip'] = fixed_ip
         router_info['ip_version'] = subnet['ip_version']
 
         try:
@@ -425,7 +354,7 @@ class AristaL3ServicePlugin(service_base.ServicePluginBase,
         router_info = copy.deepcopy(router_to_del)
         router_info['seg_id'] = seg_id
         router_info['name'] = router['name']
-        router_info['ip_version'] = subnet['ip_version']
+
         try:
             self.driver.remove_router_interface(context, router_info)
             return router_to_del
