@@ -28,7 +28,6 @@ from neutron.services.trunk import constants as trunk_consts
 
 from networking_arista.common import constants as a_const
 from networking_arista.common import db_lib
-from networking_arista.common import exceptions as arista_exc
 from networking_arista.ml2 import arista_sync
 from networking_arista.ml2 import arista_trunk
 from networking_arista.ml2.rpc.arista_eapi import AristaRPCWrapperEapi
@@ -46,11 +45,10 @@ def log_context(function, context):
 class MechResource(object):
     """Container class for passing data to sync worker"""
 
-    def __init__(self, id, resource_type, action, related_resources=None):
+    def __init__(self, id, resource_type, action):
         self.id = id
         self.resource_type = resource_type
         self.action = action
-        self.related_resources = related_resources or list()
 
     def __str__(self):
         return "%s %s ID: %s" % (self.action, self.resource_type, self.id)
@@ -80,38 +78,46 @@ class AristaDriver(driver_api.MechanismDriver):
     def get_workers(self):
         return [arista_sync.AristaSyncWorker(self.provision_queue)]
 
-    def create_network(self, network, segments):
-        """Enqueue network create"""
-        tenant_id = network['project_id']
-        action = a_const.CREATE if tenant_id else a_const.FULL_SYNC
-        n_res = MechResource(network['id'], a_const.NETWORK_RESOURCE, action)
-        n_res.related_resources.append((a_const.TENANT_RESOURCE, tenant_id))
-        for segment in segments:
-            n_res.related_resources.append(
-                (a_const.SEGMENT_RESOURCE, segment['id']))
-        self.provision_queue.put(n_res)
+    def create_tenant(self, tenant_id):
+        """Enqueue tenant create"""
+        t_res = MechResource(tenant_id, a_const.TENANT_RESOURCE,
+                             a_const.CREATE if tenant_id
+                             else a_const.FULL_SYNC)
+        self.provision_queue.put(t_res)
 
-    def delete_network(self, network, segments):
-        """Enqueue network delete"""
-        tenant_id = network['project_id']
-        action = a_const.DELETE if tenant_id else a_const.FULL_SYNC
-        n_res = MechResource(network['id'], a_const.NETWORK_RESOURCE, action)
-
-        # Delete tenant if this was the last tenant resource
+    def delete_tenant_if_removed(self, tenant_id):
+        """Enqueue tenant delete if it's no longer in the db"""
         if not db_lib.tenant_provisioned(tenant_id):
-            n_res.related_resources.append(
-                (a_const.TENANT_RESOURCE, tenant_id))
+            t_res = MechResource(tenant_id, a_const.TENANT_RESOURCE,
+                                 a_const.DELETE if tenant_id
+                                 else a_const.FULL_SYNC)
+            self.provision_queue.put(t_res)
 
-        for segment in segments:
-            n_res.related_resources.append(
-                (a_const.SEGMENT_RESOURCE, segment['id']))
+    def create_network(self, network):
+        """Enqueue network create"""
+        n_res = MechResource(network['id'], a_const.NETWORK_RESOURCE,
+                             a_const.CREATE)
         self.provision_queue.put(n_res)
 
-    def delete_segment(self, segment):
-        """Enqueue segment delete"""
-        s_res = MechResource(segment['id'], a_const.SEGMENT_RESOURCE,
+    def delete_network(self, network):
+        """Enqueue network delete"""
+        n_res = MechResource(network['id'], a_const.NETWORK_RESOURCE,
                              a_const.DELETE)
-        self.provision_queue.put(s_res)
+        self.provision_queue.put(n_res)
+
+    def create_segments(self, segments):
+        """Enqueue segment creates"""
+        for segment in segments:
+            s_res = MechResource(segment['id'], a_const.SEGMENT_RESOURCE,
+                                 a_const.CREATE)
+            self.provision_queue.put(s_res)
+
+    def delete_segments(self, segments):
+        """Enqueue segment deletes"""
+        for segment in segments:
+            s_res = MechResource(segment['id'], a_const.SEGMENT_RESOURCE,
+                                 a_const.DELETE)
+            self.provision_queue.put(s_res)
 
     def get_instance_type(self, port):
         """Determine the port type based on device owner and vnic type"""
@@ -130,6 +136,43 @@ class AristaDriver(driver_api.MechanismDriver):
             return a_const.VM_RESOURCE
         return None
 
+    def create_instance(self, port):
+        """Enqueue instance create"""
+        instance_type = self.get_instance_type(port)
+        if not instance_type:
+            return
+        i_res = MechResource(port['device_id'], instance_type, a_const.CREATE)
+        self.provision_queue.put(i_res)
+
+    def delete_instance_if_removed(self, port):
+        """Enqueue instance delete if it's no longer in the db"""
+        instance_type = self.get_instance_type(port)
+        if not instance_type:
+            return
+        if not db_lib.instance_provisioned(port['device_id']):
+            i_res = MechResource(port['device_id'], instance_type,
+                                 a_const.DELETE)
+            self.provision_queue.put(i_res)
+
+    def create_port(self, port):
+        """Enqueue port create"""
+        instance_type = self.get_instance_type(port)
+        if not instance_type:
+            return
+        port_type = instance_type + a_const.PORT_SUFFIX
+        p_res = MechResource(port['id'], port_type, a_const.CREATE)
+        self.provision_queue.put(p_res)
+
+    def delete_port_if_removed(self, port):
+        """Enqueue port delete"""
+        instance_type = self.get_instance_type(port)
+        if not instance_type:
+            return
+        port_type = instance_type + a_const.PORT_SUFFIX
+        if not db_lib.port_provisioned(port['id']):
+            p_res = MechResource(port['id'], port_type, a_const.DELETE)
+            self.provision_queue.put(p_res)
+
     def _get_binding_keys(self, port, host):
         """Get binding keys from the port binding"""
         binding_keys = list()
@@ -146,46 +189,20 @@ class AristaDriver(driver_api.MechanismDriver):
 
     def create_port_binding(self, port, host):
         """Enqueue port binding create"""
-        tenant_id = port['project_id']
-        instance_type = self.get_instance_type(port)
-        if not instance_type:
+        if not self.get_instance_type(port):
             return
-        port_type = instance_type + a_const.PORT_SUFFIX
-        action = a_const.CREATE if tenant_id else a_const.FULL_SYNC
-        related_resources = list()
-        related_resources.append((a_const.TENANT_RESOURCE, tenant_id))
-        related_resources.append((instance_type, port['device_id']))
-        related_resources.append((port_type, port['id']))
         for pb_key in self._get_binding_keys(port, host):
             pb_res = MechResource(pb_key, a_const.PORT_BINDING_RESOURCE,
-                                  action, related_resources=related_resources)
+                                  a_const.CREATE)
             self.provision_queue.put(pb_res)
 
     def delete_port_binding(self, port, host):
         """Enqueue port binding delete"""
-        tenant_id = port['project_id']
-        instance_type = self.get_instance_type(port)
-        if not instance_type:
+        if not self.get_instance_type(port):
             return
-        port_type = instance_type + a_const.PORT_SUFFIX
-        action = a_const.DELETE if tenant_id else a_const.FULL_SYNC
-        related_resources = list()
-
-        # Delete tenant if this was the last tenant resource
-        if not db_lib.tenant_provisioned(tenant_id):
-            related_resources.append((a_const.TENANT_RESOURCE, tenant_id))
-
-        # Delete instance if this was the last instance port
-        if not db_lib.instance_provisioned(port['device_id']):
-            related_resources.append((instance_type, port['device_id']))
-
-        # Delete port if this was the last port binding
-        if not db_lib.port_provisioned(port['id']):
-            related_resources.append((port_type, port['id']))
-
         for pb_key in self._get_binding_keys(port, host):
             pb_res = MechResource(pb_key, a_const.PORT_BINDING_RESOURCE,
-                                  action, related_resources=related_resources)
+                                  a_const.DELETE)
             self.provision_queue.put(pb_res)
 
     def create_network_postcommit(self, context):
@@ -195,13 +212,17 @@ class AristaDriver(driver_api.MechanismDriver):
         log_context("create_network_postcommit: network", network)
 
         segments = context.network_segments
-        self.create_network(network, segments)
+        tenant_id = network['project_id']
+        self.create_tenant(tenant_id)
+        self.create_network(network)
+        self.create_segments(segments)
 
     def update_network_postcommit(self, context):
         """Send network updates to CVX:
 
         - Update the network name
         - Add new segments
+        - Delete stale segments
         """
         network = context.current
         orig_network = context.original
@@ -211,8 +232,10 @@ class AristaDriver(driver_api.MechanismDriver):
 
         segments = context.network_segments
 
+        self.create_network(network)
+
         # New segments may have been added
-        self.create_network(network, segments)
+        self.create_segments(segments)
 
     def delete_network_postcommit(self, context):
         """Delete the network from CVX"""
@@ -221,7 +244,18 @@ class AristaDriver(driver_api.MechanismDriver):
         log_context("delete_network_postcommit: network", network)
 
         segments = context.network_segments
-        self.delete_network(network, segments)
+        tenant_id = network['project_id']
+        self.delete_segments(segments)
+        self.delete_network(network)
+        self.delete_tenant_if_removed(tenant_id)
+
+    def _delete_port_resources(self, port, host):
+        tenant_id = port['project_id']
+
+        self.delete_port_binding(port, host)
+        self.delete_port_if_removed(port)
+        self.delete_instance_if_removed(port)
+        self.delete_tenant_if_removed(tenant_id)
 
     def update_port_postcommit(self, context):
         """Send port updates to CVX
@@ -236,21 +270,27 @@ class AristaDriver(driver_api.MechanismDriver):
         log_context("update_port_postcommit: port", port)
         log_context("update_port_postcommit: orig", orig_port)
 
+        tenant_id = port['project_id']
+
         # Device id can change without a port going DOWN, but the new device
         # id may not be supported
         if orig_port and port['device_id'] != orig_port['device_id']:
-            self.delete_port_binding(orig_port, context.original_host)
+            self._delete_port_resources(orig_port, context.original_host)
 
         if context.status == n_const.PORT_STATUS_DOWN:
             if (context.original_host and
                     context.status != context.original_status):
-                self.delete_port_binding(orig_port, context.original_host)
+                self._delete_port_resources(orig_port, context.original_host)
                 self._try_to_release_dynamic_segment(context, migration=True)
         else:
+            self.create_tenant(tenant_id)
+            self.create_network(network)
             if context.binding_levels:
                 segments = [
                     level['bound_segment'] for level in context.binding_levels]
-                self.create_network(network, segments)
+                self.create_segments(segments)
+            self.create_instance(port)
+            self.create_port(port)
             self.create_port_binding(port, context.host)
 
     def delete_port_postcommit(self, context):
@@ -259,7 +299,7 @@ class AristaDriver(driver_api.MechanismDriver):
 
         log_context("delete_port_postcommit: port", port)
 
-        self.delete_port_binding(port, context.host)
+        self._delete_port_resources(port, context.host)
         self._try_to_release_dynamic_segment(context)
 
     def _bind_baremetal_port(self, context, segment):
@@ -313,7 +353,7 @@ class AristaDriver(driver_api.MechanismDriver):
                       "found", {'port': port_id})
             return False
 
-        with lockutils.lock(physnet, external=True):
+        with lockutils.lock(physnet):
             context.allocate_dynamic_segment(
                 {'network_id': context.network.current['id'],
                  'network_type': n_const.TYPE_VLAN,
@@ -329,16 +369,6 @@ class AristaDriver(driver_api.MechanismDriver):
         context.continue_binding(segment['id'], [next_segment])
         return True
 
-    def _is_in_managed_physnets(self, physnet):
-        # Check if this is a fabric segment
-        if not physnet:
-            return self.manage_fabric
-        # If managed physnet is empty, accept all.
-        if not self.managed_physnets:
-            return True
-        # managed physnet is not empty, find for matching physnet
-        return any(pn == physnet for pn in self.managed_physnets)
-
     def bind_port(self, context):
         """Bind port to a network segment.
 
@@ -394,50 +424,7 @@ class AristaDriver(driver_api.MechanismDriver):
                 continue
             if not db_lib.segment_bound(segment_id):
                 context.release_dynamic_segment(segment_id)
-                self.delete_segment(bound_segment)
+                self.delete_segments([bound_segment])
                 LOG.debug("Released dynamic segment %(seg)s allocated "
                           "by %(drv)s", {'seg': segment_id,
                                          'drv': allocating_driver})
-
-    def create_security_group(self, sg):
-        try:
-            self.eapi.create_acl(sg)
-        except Exception:
-            msg = (_('Failed to create ACL on EOS %s') % sg)
-            LOG.exception(msg)
-            raise arista_exc.AristaSecurityGroupError(msg=msg)
-
-    def delete_security_group(self, sg):
-        try:
-            self.eapi.delete_acl(sg)
-        except Exception:
-            msg = (_('Failed to create ACL on EOS %s') % sg)
-            LOG.exception(msg)
-            raise arista_exc.AristaSecurityGroupError(msg=msg)
-
-    def update_security_group(self, sg):
-        try:
-            self.eapi.create_acl(sg)
-        except Exception:
-            msg = (_('Failed to create ACL on EOS %s') % sg)
-            LOG.exception(msg)
-            raise arista_exc.AristaSecurityGroupError(msg=msg)
-
-    def create_security_group_rule(self, sgr):
-        try:
-            self.eapi.create_acl_rule(sgr)
-        except Exception:
-            msg = (_('Failed to create ACL rule on EOS %s') % sgr)
-            LOG.exception(msg)
-            raise arista_exc.AristaSecurityGroupError(msg=msg)
-
-    def delete_security_group_rule(self, sgr_id):
-        if sgr_id:
-            sgr = self.ndb.get_security_group_rule(sgr_id)
-            if sgr:
-                try:
-                    self.eapi.delete_acl_rule(sgr)
-                except Exception:
-                    msg = (_('Failed to delete ACL rule on EOS %s') % sgr)
-                    LOG.exception(msg)
-                    raise arista_exc.AristaSecurityGroupError(msg=msg)
