@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections import defaultdict
 import json
 import re
 
@@ -30,6 +31,7 @@ from neutron.services.trunk import constants as t_const
 from neutron.services.trunk import models as t_models
 
 from networking_arista.common import config  # noqa
+from networking_arista.common import constants as a_const
 from networking_arista.ml2 import arista_resources as resources
 
 LOG = logging.getLogger(__name__)
@@ -41,7 +43,6 @@ def setup_arista_wrapper_config(cfg, host='host', user='user'):
     cfg.CONF.set_override('sync_interval', 1, "ml2_arista")
     cfg.CONF.set_override('conn_timeout', 20, "ml2_arista")
     cfg.CONF.set_override('switch_info', ['switch1:user:pass'], "ml2_arista")
-    cfg.CONF.set_override('sec_group_support', False, "ml2_arista")
 
 
 class MockCvx(object):
@@ -98,6 +99,8 @@ class MockSwitch(object):
 
     def __init__(self):
         self._commands = []
+        self._acl_rules = dict()
+        self._bindings = defaultdict(lambda: defaultdict(set))
         self._vrfs = dict()
         self._svis = dict()
         self._vlans = dict()
@@ -108,7 +111,7 @@ class MockSwitch(object):
         self._access_group_re = re.compile(
             '^(?P<delete>no )?ip access-group (?P<acl>\S+) (?P<dir>\S+)$')
         self._vrf_mode_re = re.compile(
-            '^(?P<delete>no )?vrf instance (?P<name>\S+)$')
+            '^(?P<delete>no )?vrf definition (?P<name>\S+)$')
         self._vlan_re = re.compile('^(?P<delete>no )?vlan (?P<vlan>\d+)$')
         self._ip_address_re = re.compile(
             '^ip address (?P<ip>[\d.]+)/(?P<mask>\d+)$')
@@ -119,10 +122,29 @@ class MockSwitch(object):
             '^ip virtual-router mac-address (?P<varp_mac>\S+)$')
         self._mode = None
 
-    def execute(self, commands, commands_to_log=None, keep_alive=True):
+    def execute(self, commands, commands_to_log=None):
         ret = []
         for command in commands:
-            if command == 'show vlan':
+            if command == 'show ip access-lists dynamic':
+                acls = {'aclList': []}
+                for name, rules in self._acl_rules.items():
+                    acls['aclList'].append(
+                        {'name': name,
+                         'sequence': [{'text': rule} for rule in rules]})
+                ret.append(acls)
+            elif command == 'show ip access-lists summary dynamic':
+                bindings = {'aclList': []}
+                for acl, binding in self._bindings.items():
+                    bindings['aclList'].append(
+                        {'name': acl,
+                         'configuredIngressIntfs': [
+                             {'name': intf} for intf in
+                             binding.get(a_const.INGRESS_DIRECTION, [])],
+                         'configuredEgressIntfs': [
+                             {'name': intf} for intf in
+                             binding.get(a_const.EGRESS_DIRECTION, [])]})
+                ret.append(bindings)
+            elif command == 'show vlan':
                 vlans = {'vlans': {}}
                 for vlan, info in self._vlans.items():
                     vlans['vlans'][str(vlan)] = {'dynamic': info['dynamic']}
@@ -141,12 +163,21 @@ class MockSwitch(object):
                     vrfs['vrfs'][vrf_name] = {'interfaces': vrf['svis'],
                                               'routeDistinguisher': vrf['rd']}
                 ret.append(vrfs)
-            elif command == 'show version':
-                ret.append({'version': '4.22.0F'})
             elif command == 'enable':
                 ret.append({})
             elif 'show' in command:
                 pass
+            elif 'ip access-list' in command:
+                acl_match = self._acl_mode_re.match(command)
+                acl = acl_match.group('acl')
+                delete = acl_match.group('delete')
+                if delete:
+                    del self._acl_rules[acl]
+                else:
+                    assert acl_match.group('dyn'), "All ACLs must be dynamic"
+                    self._mode = ('acl', acl)
+                    if not self._acl_rules.get(acl):
+                        self._acl_rules[acl] = list()
             elif 'interface' in command:
                 intf_match = self._interface_mode_re.match(command)
                 intf = intf_match.group('intf')
@@ -158,7 +189,7 @@ class MockSwitch(object):
                                             'mask': '',
                                             'vip': ''}
                     self._mode = ('interface', intf)
-            elif 'vrf instance' in command:
+            elif 'vrf definition' in command:
                 vrf_match = self._vrf_mode_re.match(command)
                 delete = vrf_match.group('delete')
                 vrf_name = vrf_match.group('name')
@@ -173,13 +204,36 @@ class MockSwitch(object):
                 self._mode = None
             else:
                 if self._mode:
-                    if self._mode[0] == 'interface':
-                        self._parse_svi(command)
+                    if self._mode[0] == 'acl':
+                        if command.startswith('no '):
+                            self._acl_rules[self._mode[1]].remove(command[3:])
+                        else:
+                            self._acl_rules[self._mode[1]].append(command)
+                    elif self._mode[0] == 'interface':
+                        acl_match = self._access_group_re.match(command)
+                        if acl_match:
+                            self._parse_acl(acl_match)
+                        else:
+                            self._parse_svi(command)
                     elif self._mode[0] == 'vrf':
                         self._parse_vrf(command)
             self._commands.append(command)
         return ret
 
+    def _parse_acl(self, acl_match):
+        delete = acl_match.group('delete')
+        acl = acl_match.group('acl')
+        direction = acl_match.group('dir')
+        if delete:
+            self._bindings[acl][direction].remove(
+                self._mode[1])
+        else:
+            # Delete the old binding for this intf if nec.
+            for binding in self._bindings.values():
+                if self._mode[1] in binding[direction]:
+                    binding[direction].remove(self._mode[1])
+            self._bindings[acl][direction].add(self._mode[1])
+
     def _parse_svi(self, command):
         ip_addr_match = self._ip_address_re.match(command)
         if ip_addr_match:
@@ -218,6 +272,8 @@ class MockSwitch(object):
 
     def reset_switch(self):
         self._commands = []
+        self._acl_rules = dict()
+        self._bindings = defaultdict(lambda: defaultdict(set))
         self._vrfs = dict()
         self._svis = dict()
         self._vlans = dict()
